/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.crop.type;

import java.io.IOException;
import java.io.ObjectOutputStream;
import nl.cloudfarming.client.crop.model.Crop;
import nl.cloudfarming.client.crop.model.CropService;
import nl.cloudfarming.client.crop.model.CropsStorage;
import nl.cloudfarming.client.crop.model.Genus;
import nl.cloudfarming.client.crop.model.Race;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;
import org.openide.util.lookup.ServiceProvider;

/**
 * Service for getting and adding crops to a certain project
 * @author Merijn Zengers
 */
@ServiceProvider(service = CropService.class)
public final class CropServiceImpl implements CropService {

    protected static final String CROP_DIR = "crops";
    protected static final String CROP_PREFIX = ".crp";
    protected static final String NEW_CROP_FILE_NAME = "new";
    
    public CropServiceImpl(){
    }

    /**
     * Gets crops from the specified Project
     * If there are no crops in the project the method will create two example crops
     * @param project The project which contains the crops
     * @return List of found CropDataObjects
     */
    @Override
    public CropsStorageDataObject getCropsStorage(Project project) throws DataObjectNotFoundException, IOException {
        FileObject cropFolder = getCropFolder(project.getProjectDirectory(), true);
        if (cropFolder.getChildren().length == 0) {
            //Add two default crops to the directory
            generateCrops(cropFolder);
        }
        for (FileObject file : cropFolder.getChildren()) {
            DataObject found = DataObject.find(file);
            if (found instanceof CropsStorageDataObject) {
                return (CropsStorageDataObject) found;
            }
        }
        return null;
    }

    /**
     * Method to get the crop folder containing crops
     * @param projectDir the projectDir the crop directory should reside in
     * @param create if not found create the folder
     * @return the found or created FileObject
     */
    private FileObject getCropFolder(FileObject projectDir, boolean create) {
        FileObject result = projectDir.getFileObject(CROP_DIR);
        if (result == null && create) {
            try {
                result = projectDir.createFolder(CROP_DIR);
            } catch (IOException ioe) {
                Exceptions.printStackTrace(ioe);
            }
        }
        return result;
    }

    /**
     * Method to load a few default crops into a project
     * @param dir the dir to place the crops
     * @throws IOException
     */
    @Messages({"crop.genus.triticum_aestivum.winter.name=Winter wheat", "crop.race.triticum_aestivum.alcyon=Alcyon", "crop.genus.triticum_aestivum.spring.name=Spring wheat", "crop.race.triticum_aestivum.alexandria=Alexandria", "crop.genus=Class 1"})
    private void generateCrops(FileObject dir) throws IOException {
        final String CROPFILE_WINTER_WHEAT = "crops.crp";

        final String GENUS = NbBundle.getMessage(CropServiceImpl.class, "crop.genus");
        final String WINTER_WHEAT_CROP = NbBundle.getMessage(CropServiceImpl.class, "crop.genus.triticum_aestivum.winter.name");
        final String WINTER_WHEAT_TRITICUM_AESTIVUM_ALCUON = NbBundle.getMessage(CropServiceImpl.class, "crop.race.triticum_aestivum.alcyon");

        final String SPRING_WHEAT_CROP = NbBundle.getMessage(CropServiceImpl.class, "crop.genus.triticum_aestivum.spring.name");
        final String SPRING_WHEAT_TRITICUM_AESTIVUM_ALCUON = NbBundle.getMessage(CropServiceImpl.class, "crop.race.triticum_aestivum.alexandria");
        
        
        try (ObjectOutputStream oos = new ObjectOutputStream(dir.createAndOpen(CROPFILE_WINTER_WHEAT));) {
            CropsStorage cropsStorage = new CropsStorage();
            
            Crop crop = new Crop();
            crop.setName(WINTER_WHEAT_CROP);
            Race winterWheatTriticum = new Race();
            winterWheatTriticum.setName(WINTER_WHEAT_TRITICUM_AESTIVUM_ALCUON);
            Genus genus = new Genus();
            genus.setName(GENUS);
            crop.addGenus(genus);
            crop.addRace(winterWheatTriticum);

            
            Crop crop2 = new Crop();
            crop2.setName(SPRING_WHEAT_CROP);
            Race springWheatTriticum = new Race();
            springWheatTriticum.setName(SPRING_WHEAT_TRITICUM_AESTIVUM_ALCUON);
            crop2.addGenus(genus);
            crop2.addRace(springWheatTriticum);
            
            cropsStorage.addCrop(crop);
            cropsStorage.addCrop(crop2);
            
            oos.writeObject(cropsStorage);   
        }
    }
}
