/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.crop.type;

import nl.cloudfarming.client.crop.model.Crop;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.logging.Logger;
import nl.cloudfarming.client.crop.model.CropManager;
import nl.cloudfarming.client.crop.model.CropsStorage;
import nl.cloudfarming.client.crop.model.Genus;
import nl.cloudfarming.client.crop.model.Race;
import org.openide.awt.UndoRedo;
import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

/**
 * Data object representing a crop object.
 * On construction it will try to de-serialize the provided FileObject  to an crop object.
 * Registers a listener on the Crop object to listen to changes.
 * On change of the Crop object will add a save cookie to its InstanceContent.
 * If the save action is performed on the cookie it will perform save on the CropManager registered as service provider of CropManager
 * @author Merijn Zengers
 */
public class CropsStorageDataObject extends MultiDataObject implements PropertyChangeListener {

    private UndoRedo.Manager manager;
    private CropsStorage cropsStorage;
    private InstanceContent ic;
    private Lookup intern;
    private CropManager cropManager;
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.crop.type");
    public static final String CHANGED = "changed";

    public CropsStorageDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor("application/x-agrosense-crop", true);

        try (ObjectInputStream ois = new ObjectInputStream(pf.getInputStream())) {
            cropsStorage = (CropsStorage) ois.readObject();
            this.ic = new InstanceContent();
            this.intern = new AbstractLookup(this.ic);

            cropManager = Lookup.getDefault().lookup(CropManager.class);
            attachListeners(cropsStorage);


        } catch (ClassNotFoundException ex) {
            Exceptions.printStackTrace(ex);
        }
    }

    /**
     * Method to attach property changeListeners and undo redo managers too crop
     * To the races of the crop
     * And to the genus of the race
     * @param crop 
     */
    protected final void attachListeners(CropsStorage crops) {
        crops.addUndoableEditListener(getUndoRedo());
        crops.addPropertyChangeListener(this);
        for (Crop crop : crops.getCrops()) {
            crop.addPropertyChangeListener(this);
            crop.addUndoableEditListener(getUndoRedo());
            for (Race race : crop.getRaces()) {
                race.addPropertyChangeListener(this);
                race.addUndoableEditListener(getUndoRedo());
            }
            for (Genus genus : crop.getGenera()) {
                genus.addPropertyChangeListener(this);
                genus.addUndoableEditListener(getUndoRedo());
            }
        }
    }

    /**
     * Returns the CropsStorage Object belonging too this CropsStorageDataObject
     * The CropsStorage Object is read from a serialized file
     * @return the CropsStorage
     */
    public CropsStorage getCropsStorage() {
        return cropsStorage;
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), intern);
    }
    
    public UndoRedo.Manager getUndoRedo() {
        if(manager == null){
            manager = new UndoRedo.Manager();
        }
        return manager;
    }

    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        if (this.getLookup() != null) {
            SaveCookie saveCookie = this.getLookup().lookup(SaveCookie.class);
            if (null == saveCookie) {
                this.ic.add(new CropSaveCapability());
            }
            //Adding
            if (pce.getNewValue() != null && pce.getOldValue() == null) {
                if (pce.getPropertyName().equals(Crop.PROP_RACE)) {
                    //A Race is added Attach property listeners to the new race 
                    Race newRace = (Race) pce.getNewValue();
                    newRace.addPropertyChangeListener(this);
                } else if (pce.getPropertyName().equals(Crop.PROP_GENUS)) {
                    //A Genus is added Attach property listeners to the new genus 
                    Genus genus = (Genus) pce.getNewValue();
                    genus.addPropertyChangeListener(this);
                } else if (pce.getPropertyName().equals(CropsStorage.PROP_CROP)) {
                    Crop crop = (Crop) pce.getNewValue();
                    crop.addPropertyChangeListener(this);
                }
                //Removing
            } else if (pce.getNewValue() == null && pce.getOldValue() != null) {
                if (pce.getPropertyName().equals(Crop.PROP_RACE)) {
                    Race removedRace = (Race) pce.getOldValue();
                    removedRace.removePropertyChangeListener(this);
                } else if (pce.getPropertyName().equals(Crop.PROP_GENUS)) {
                    Genus removedGenus = (Genus) pce.getOldValue();
                    removedGenus.removePropertyChangeListener(this);
                } else if (pce.getPropertyName().equals(CropsStorage.PROP_CROP)) {
                    Crop crop = (Crop) pce.getOldValue();
                    crop.removePropertyChangeListener(this);
                }
            }
        }
    }

    private class CropSaveCapability implements SaveCookie {

        @Override
        public void save() throws IOException {
            FileObject fo = CropsStorageDataObject.this.getPrimaryFile();
            CropsStorage crops = CropsStorageDataObject.this.getCropsStorage();
            CropsStorageDataObject.this.cropManager.save(crops, fo);
            CropsStorageDataObject.this.ic.remove(CropSaveCapability.this);
        }
    }
}
